"""
Copyright 2019 Cognitive Scale, Inc. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import urllib.parse

from .utils import get_logger
from .serviceconnector import _Client
from .camel import CamelResource
from .utils import raise_for_status_with_detail

log = get_logger(__name__)


class SkillClient(_Client):
    """
    A client for the Cortex Skill management API.
    """

    URIs = {
        'skills': 'projects/{projectId}/skills',
        'skill': 'projects/{projectId}/skills/{skillName}',
        'logs': 'projects/{projectId}/skills/{skillName}/action/{actionName}/logs',
        'deploy': 'projects/{projectId}/skills/{skillName}/deploy',
        'undeploy': 'projects/{projectId}/skills/{skillName}/undeploy'

    }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._serviceconnector.version = 4
        if len(args) > 0:
            self._project = args[0]._project
        else:
            self._project = None

    def list_skills(self):
        r = self._serviceconnector.request(method='GET', uri=self.URIs['skills'].format(projectId=self._project))
        raise_for_status_with_detail(r)
        rs = r.json()

        return rs.get('skills', [])

    def save_skill(self, skill_obj):
        body = json.dumps(skill_obj)
        headers = {'Content-Type': 'application/json'}
        uri = self.URIs['skills'].format(projectId=self._project)
        r = self._serviceconnector.request(method='POST', uri=uri, body=body, headers=headers)
        raise_for_status_with_detail(r)
        return r.json()

    def get_skill(self, skill_name):
        uri = self.URIs['skill'].format(projectId=self._project, skillName=self.parse_string(skill_name))
        r = self._serviceconnector.request(method='GET', uri=uri)
        raise_for_status_with_detail(r)

        return r.json()

    def delete_skill(self, skill_name):
        uri = self.URIs['skill'].format(projectId=self._project, skillName=self.parse_string(skill_name))
        r = self._serviceconnector.request(method='DELETE', uri=uri)
        raise_for_status_with_detail(r)
        rs = r.json()

        return rs.get('success', False)

    def get_logs(self, skill_name, action_name):
        uri = self.URIs['logs'].format(projectId=self._project, skillName=self.parse_string(skill_name),
                                       actionName=self.parse_string(action_name))
        r = self._serviceconnector.request(method='GET', uri=uri)
        raise_for_status_with_detail(r)

        return r.json()

    def deploy(self, skill_name):
        uri = self.URIs['deploy'].format(projectId=self._project, skillName=self.parse_string(skill_name))
        r = self._serviceconnector.request(method='GET', uri=uri)
        raise_for_status_with_detail(r)

        return r.json()

    def undeploy(self, skill_name):
        uri = self.URIs['undeploy'].format(projectId=self._project, skillName=self.parse_string(skill_name))
        r = self._serviceconnector.request(method='GET', uri=uri)
        raise_for_status_with_detail(r)

        return r.json()

    def parse_string(self, string):
        # Replaces special characters like / with %2F
        return urllib.parse.quote(string, safe='')


class Skill(CamelResource):
    """
     Computational components of an agent; executes an
     atomic unit of work and can be triggered by one or more inputs to produce
     one or more outputs.
    """
    def __init__(self, skill, project: str, client: SkillClient):
        super().__init__(skill, True)
        self._client = client
        self._project = project



"""
Copyright 2019 Cognitive Scale, Inc. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import urllib.parse

from .camel import CamelResource
from typing import Dict
from .serviceconnector import _Client
from .utils import raise_for_status_with_detail


class ModelClient(_Client):

    """
    A client for the Cortex model management API.
    """

    URIs = {
        'models': 'projects/{projectId}/models',
        'model': 'projects/{projectId}/models/{modelName}'

    }

    def __init__(self, project, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._serviceconnector.version = 4
        self._project = project

    def list_models(self):
        r = self._serviceconnector.request(method='GET', uri=self.URIs['models'].format(projectId=self._project))
        raise_for_status_with_detail(r)
        rs = r.json()

        return rs.get('models', [])

    def save_model(self, model_obj):
        body = json.dumps(model_obj)
        headers = {'Content-Type': 'application/json'}
        uri = self.URIs['models'].format(projectId=self._project)
        r = self._serviceconnector.request(method='POST', uri=uri, body=body, headers=headers)
        raise_for_status_with_detail(r)
        return r.json()

    def get_model(self, model_name):
        uri = self.URIs['model'].format(projectId=self._project, modelName=self.parse_string(model_name))
        r = self._serviceconnector.request(method='GET', uri=uri)
        raise_for_status_with_detail(r)

        return r.json()

    def delete_model(self, model_name):
        uri = self.URIs['model'].format(projectId=self._project, modelName=self.parse_string(model_name))
        r = self._serviceconnector.request(method='DELETE', uri=uri)
        raise_for_status_with_detail(r)
        rs = r.json()

        return rs.get('success', False)

    def parse_string(self, string):
        # Replaces special characters like / with %2F
        return urllib.parse.quote(string, safe='')


class Model(CamelResource):
    """
    Tracks associated parameters of models.
    """

    def __init__(self, document: Dict, project: str, client: ModelClient):
        super().__init__(document, False)
        self._project = project
        self._client = client

    def to_camel(self, camel='1.0.0'):
        return {
            'camel': camel,
            'name': self.name,
            'title': self.title,
            'description': self.description,
            'status': self.status,
            'type': self.type,
            'tags': self.tags or [],
            'properties': self.properties or [],
        }



'''
functions for mocking connection to cortex for testing 
'''

import json
from cortex.utils import generate_token
from mocket.mockhttp import Entry

def john_doe_token():
    return generate_token(mock_pat_config())

def john_doe_subject():
    '''
    The subject part of the jwt token
    '''
    return '71a8faac-9dfb-428d-a90c-0b53481b8665'


def build_mock_url(uri, version=4):
    '''
    build a mock url for testing
    '''
    return "{api_endpoint}/fabric/v{version}/{uri}".format(api_endpoint=mock_api_endpoint(), version=version, uri=uri)


def mock_api_endpoint():
    '''
    the url endppoint for mocking
    '''
    return 'http://1.2.3.4:8000'


def register_entry(verb, url, body:dict):
    print('Registering mock for', verb, url)
    Entry.single_register(verb, url, status=200, body=json.dumps(body))


def register_entry_from_path(verb, url, path:str):
    with open(path) as fh:
        register_entry(verb, url, json.load(fh))

def mock_pat_config():
    '''
    Return a PAt config to allow test t ocreate new JWTs
    :return: PAT config
    '''
    return {
        "jwk":{
            "crv":"Ed25519",
            "x":"fPxYvREALYwHVpZtCmYxtc2EW6asDTpqyhqgfco6kWQ",
            "d":"B9vmiGHtwG9GuFZNso3JDm5_O9bcUbtQyGgp6VrAfL4",
            "kty":"OKP","kid":"HpW-ya7FSU7yV-alzyewPPDwPeFgrki0VQPKbh4J4Pw"
        },
        "issuer":"cognitivescale.com",
        "audience":"cortex",
        "username":"71a8faac-9dfb-428d-a90c-0b53481b8665",
        "url":"https://192.168.39.27:31326"
    }
